@@ -1,4 +1,3 @@
-# *** encoding: utf-8 ***
 """
 An :class:`~.KeyProcessor` receives callbacks for the keystrokes parsed from
 the input in the :class:`~prompt_toolkit.inputstream.InputStream` instance.
@@ -6,10 +5,12 @@ the input in the :class:`~prompt_toolkit.inputstream.InputStream` instance.
 The `KeyProcessor` will according to the implemented keybindings call the
 correct callbacks when new key presses are feed through `feed`.
 """
+from __future__ import annotations
+
 import weakref
 from asyncio import Task, sleep
 from collections import deque
-from typing import TYPE_CHECKING, Any, Deque, Generator, List, Optional, Union
+from typing import TYPE_CHECKING, Any, Deque, Generator
 
 from prompt_toolkit.application.current import get_app
 from prompt_toolkit.enums import EditingMode
@@ -37,7 +38,7 @@ class KeyPress:
     :param data: The received string on stdin. (Often vt100 escape codes.)
     """
 
-    def __init__(self, key: Union[Keys, str], data: Optional[str] = None) -> None:
+    def __init__(self, key: Keys | str, data: str | None = None) -> None:
         assert isinstance(key, Keys) or len(key) == 1
 
         if data is None:
@@ -50,7 +51,7 @@ class KeyPress:
         self.data = data
 
     def __repr__(self) -> str:
-        return "%s(key=%r, data=%r)" % (self.__class__.__name__, self.key, self.data)
+        return f"{self.__class__.__name__}(key={self.key!r}, data={self.data!r})"
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, KeyPress):
@@ -93,30 +94,30 @@ class KeyProcessor:
         self.before_key_press = Event(self)
         self.after_key_press = Event(self)
 
-        self._flush_wait_task: Optional[Task] = None
+        self._flush_wait_task: Task[None] | None = None
 
         self.reset()
 
     def reset(self) -> None:
-        self._previous_key_sequence: List[KeyPress] = []
-        self._previous_handler: Optional[Binding] = None
+        self._previous_key_sequence: list[KeyPress] = []
+        self._previous_handler: Binding | None = None
 
         # The queue of keys not yet send to our _process generator/state machine.
         self.input_queue: Deque[KeyPress] = deque()
 
         # The key buffer that is matched in the generator state machine.
         # (This is at at most the amount of keys that make up for one key binding.)
-        self.key_buffer: List[KeyPress] = []
+        self.key_buffer: list[KeyPress] = []
 
         #: Readline argument (for repetition of commands.)
         #: https://www.gnu.org/software/bash/manual/html_node/Readline-Arguments.html
-        self.arg: Optional[str] = None
+        self.arg: str | None = None
 
         # Start the processor coroutine.
         self._process_coroutine = self._process()
         self._process_coroutine.send(None)  # type: ignore
 
-    def _get_matches(self, key_presses: List[KeyPress]) -> List[Binding]:
+    def _get_matches(self, key_presses: list[KeyPress]) -> list[Binding]:
         """
         For a list of :class:`KeyPress` instances. Give the matching handlers
         that would handle this.
@@ -126,7 +127,7 @@ class KeyProcessor:
         # Try match, with mode flag
         return [b for b in self._bindings.get_bindings_for_keys(keys) if b.filter()]
 
-    def _is_prefix_of_longer_match(self, key_presses: List[KeyPress]) -> bool:
+    def _is_prefix_of_longer_match(self, key_presses: list[KeyPress]) -> bool:
         """
         For a list of :class:`KeyPress` instances. Return True if there is any
         handler that is bound to a suffix of this keys.
@@ -137,9 +138,9 @@ class KeyProcessor:
         # Note that we transform it into a `set`, because we don't care about
         # the actual bindings and executing it more than once doesn't make
         # sense. (Many key bindings share the same filter.)
-        filters = set(
+        filters = {
             b.filter for b in self._bindings.get_bindings_starting_with_keys(keys)
-        )
+        }
 
         # When any key binding is active, return True.
         return any(f() for f in filters)
@@ -215,7 +216,7 @@ class KeyProcessor:
         else:
             self.input_queue.append(key_press)
 
-    def feed_multiple(self, key_presses: List[KeyPress], first: bool = False) -> None:
+    def feed_multiple(self, key_presses: list[KeyPress], first: bool = False) -> None:
         """
         :param first: If true, insert before everything else.
         """
@@ -255,12 +256,9 @@ class KeyProcessor:
             else:
                 return self.input_queue.popleft()
 
-        keys_processed = False
         is_flush = False
 
         while not_empty():
-            keys_processed = True
-
             # Process next key.
             key_press = get_next()
 
@@ -277,21 +275,16 @@ class KeyProcessor:
                 # an exception was raised) restart the processor for next time.
                 self.reset()
                 self.empty_queue()
-                app.invalidate()
                 raise
 
             if not is_flush and not is_cpr:
                 self.after_key_press.fire()
 
-        if keys_processed:
-            # Invalidate user interface.
-            app.invalidate()
-
         # Skip timeout if the last key was flush.
         if not is_flush:
             self._start_timeout()
 
-    def empty_queue(self) -> List[KeyPress]:
+    def empty_queue(self) -> list[KeyPress]:
         """
         Empty the input queue. Return the unprocessed input.
         """
@@ -302,7 +295,7 @@ class KeyProcessor:
         key_presses = [k for k in key_presses if k.key != Keys.CPRResponse]
         return key_presses
 
-    def _call_handler(self, handler: Binding, key_sequence: List[KeyPress]) -> None:
+    def _call_handler(self, handler: Binding, key_sequence: list[KeyPress]) -> None:
         app = get_app()
         was_recording_emacs = app.emacs_state.is_recording
         was_recording_vi = bool(app.vi_state.recording_register)
@@ -353,7 +346,7 @@ class KeyProcessor:
                 for k in key_sequence:
                     app.vi_state.current_recording += k.data
 
-    def _fix_vi_cursor_position(self, event: "KeyPressEvent") -> None:
+    def _fix_vi_cursor_position(self, event: KeyPressEvent) -> None:
         """
         After every command, make sure that if we are in Vi navigation mode, we
         never put the cursor after the last character of a line. (Unless it's
@@ -374,7 +367,7 @@ class KeyProcessor:
             # (This was cleared after changing the cursor position.)
             buff.preferred_column = preferred_column
 
-    def _leave_vi_temp_navigation_mode(self, event: "KeyPressEvent") -> None:
+    def _leave_vi_temp_navigation_mode(self, event: KeyPressEvent) -> None:
         """
         If we're in Vi temporary navigation (normal) mode, return to
         insert/replace mode after executing one action.
@@ -401,7 +394,7 @@ class KeyProcessor:
             return
 
         async def wait() -> None:
-            " Wait for timeout. "
+            "Wait for timeout."
             # This sleep can be cancelled. In that case we don't flush.
             await sleep(timeout)
 
@@ -410,7 +403,7 @@ class KeyProcessor:
                 flush_keys()
 
         def flush_keys() -> None:
-            " Flush keys. "
+            "Flush keys."
             self.feed(_Flush)
             self.process_keys()
 
@@ -419,6 +412,13 @@ class KeyProcessor:
             self._flush_wait_task.cancel()
         self._flush_wait_task = app.create_background_task(wait())
 
+    def send_sigint(self) -> None:
+        """
+        Send SIGINT. Immediately call the SIGINT key handler.
+        """
+        self.feed(KeyPress(key=Keys.SIGINT), first=True)
+        self.process_keys()
+
 
 class KeyPressEvent:
     """
@@ -433,13 +433,12 @@ class KeyPressEvent:
 
     def __init__(
         self,
-        key_processor_ref: "weakref.ReferenceType[KeyProcessor]",
-        arg: Optional[str],
-        key_sequence: List[KeyPress],
-        previous_key_sequence: List[KeyPress],
+        key_processor_ref: weakref.ReferenceType[KeyProcessor],
+        arg: str | None,
+        key_sequence: list[KeyPress],
+        previous_key_sequence: list[KeyPress],
         is_repeat: bool,
     ) -> None:
-
         self._key_processor_ref = key_processor_ref
         self.key_sequence = key_sequence
         self.previous_key_sequence = previous_key_sequence
@@ -451,7 +450,7 @@ class KeyPressEvent:
         self._app = get_app()
 
     def __repr__(self) -> str:
-        return "KeyPressEvent(arg=%r, key_sequence=%r, is_repeat=%r)" % (
+        return "KeyPressEvent(arg={!r}, key_sequence={!r}, is_repeat={!r})".format(
             self.arg,
             self.key_sequence,
             self.is_repeat,
@@ -469,14 +468,14 @@ class KeyPressEvent:
         return processor
 
     @property
-    def app(self) -> "Application[Any]":
+    def app(self) -> Application[Any]:
         """
         The current `Application` object.
         """
         return self._app
 
     @property
-    def current_buffer(self) -> "Buffer":
+    def current_buffer(self) -> Buffer:
         """
         The current buffer.
         """
@@ -520,11 +519,11 @@ class KeyPressEvent:
         elif current is None:
             result = data
         else:
-            result = "%s%s" % (current, data)
+            result = f"{current}{data}"
 
         self.key_processor.arg = result
 
     @property
-    def cli(self) -> "Application":
-        " For backward-compatibility. "
+    def cli(self) -> Application[Any]:
+        "For backward-compatibility."
         return self.app
