@@ -15,7 +15,7 @@ container object.
 from __future__ import annotations
 
 from functools import partial
-from typing import Callable, Generic, List, Optional, Sequence, Tuple, TypeVar, Union
+from typing import Callable, Generic, Sequence, TypeVar
 
 from prompt_toolkit.application.current import get_app
 from prompt_toolkit.auto_suggest import AutoSuggest, DynamicAutoSuggest
